"""
Module defining environments in which the Vega market sim can be interacted with by
prespecified Agent classes. This enables a modular approach to simulating a
market behaviour, be that with Agents with pre-specified strategies or
those which learn through interaction with Vega itself.

Currently there are two major formats of Environment, the MarketEnvironment
and the MarketEnvironmentWithState. The crucial difference between these
two is that in the *WithState version each agent recieves, at each timestep, a
summary object representing the network's state at that time. This is
intended to eventually speed up the step process overall as individual agents
do not each need to gather state. Currently this 'state' object is a work-in-progress.

The alternative environment, plain MarketEnvironment, passes the VegaService
itself to agents at each timestep, allowing them to gather their own
information. This allows more 'custom' agents but may be ultimately slower
if the same information is retrieved multiple times.

For examples of this setup, see examples/agent_market.

"""

import grpc
import time
import datetime
import logging
import random
from collections import namedtuple
from typing import Any, Callable, Dict, List, Optional

from vega_sim.environment.agent import Agent, StateAgent, VegaState
from vega_sim.network_service import VegaServiceNetwork
from vega_sim.null_service import VegaServiceNull
from vega_sim.service import VegaService

logger = logging.getLogger(__name__)

MarketState = namedtuple(
    "MarketState",
    [
        "state",
        "trading_mode",
        "midprice",
        "orders",
    ],  # "order_book"]
)


class MarketEnvironment:
    def __init__(
        self,
        agents: List[Agent],
        n_steps: int,
        random_agent_ordering: bool = True,
        transactions_per_block: int = 1,
        block_length_seconds: int = 1,
        step_length_seconds: Optional[int] = None,
        vega_service: Optional[VegaServiceNull] = None,
        state_extraction_fn: Optional[
            Callable[[VegaServiceNull, List[Agent]], Any]
        ] = None,
        state_extraction_freq: int = 10,
        pause_every_n_steps: Optional[int] = None,
    ):
        """Set up a Vega protocol environment with some specified agents.
        Handles the entire Vega setup and environment lifetime process, allowing the
        user to focus on building the Agents themselves.

        Once an environment has been created, calling the 'run' function will
        run a complete simulation of the environment lifecycle then shut down
        the various Vega processes.

        Args:
            agents:
                List[Agent], a list of instantiated Agent objects which will
                    interact with the environment
            n_steps:
                int, The number of steps for the environment to run
            random_agent_ordering:
                bool, default True, In each step, whether the order of agent
                    steps should be randomised. If False, the order of agents
                    passed in will be used
            transactions_per_block:
                int, default 1, How many transactions should be contained
                    for each block in the Vega chain. Often this is best set
                    as the maximum number of actions agents can take per step
                    to ensure they all happen 'at the same time' per step.
            block_length_seconds:
                int, default 1, How many seconds each block on the Vega chain
                    represents
            step_length_seconds:
                Optional[int], default None, How many seconds each step is
                    taken to represent.
                    After each round of actions, if time has not advanced at least
                    this much we will be forwarded to that far in the future
                    (minus however long the actions did take).
                    e.g. for a step_length_seconds = 60 if all actions take up 10s
                    we will forward 50s at the end.
            vega_service:
                optional, VegaServiceNull, If passed will use this precreated vega
                    service instead of creating one internally.
            state_extraction_fn:
                optional, Callable[[VegaServiceNull, List[Agent]], Any],
                    Optional function which takes a Vega service at a given time
                    and returns a summary of interesting state values. If passed,
                    the aggregated result of these will be returned after a run.
            state_extraction_freq:
                int, default 10, If state_extraction_fn is passed, how many steps
                    should be between each call.
            pause_every_n_steps:
                Optional[int], default None, If passed, simulation will pause every
                    time the passed number of steps elapses waiting on user to press
                    return. Allows inspection of the simulation at given frequency
        """
        self.agents = agents
        self.n_steps = n_steps
        self.random_agent_ordering = random_agent_ordering
        self.transactions_per_block = transactions_per_block
        self.block_length_seconds = block_length_seconds
        self.step_length_seconds = step_length_seconds
        self._vega = vega_service
        self._state_extraction_fn = state_extraction_fn
        self._state_extraction_freq = state_extraction_freq
        self._pause_every_n_steps = pause_every_n_steps

    def run(
        self,
        run_with_console: bool = False,
        pause_at_completion: bool = False,
    ) -> Optional[List[Any]]:
        """Run the simulation with specified agents.

        Args:
            run_with_console:
                bool, default False, Whether the environment should attempt
                    to spin up a full Vega console with which to observe
                    the market behaviour
            pause_at_completion:
                bool, default False, If True will pause with a keypress-prompt
                    once the simulation has completed, allowing the final state
                    to be inspected, either via code or the Console
        """
        if self._vega is None:
            with VegaServiceNull(
                run_with_console=run_with_console,
                warn_on_raw_data_access=False,
                transactions_per_block=self.transactions_per_block,
                block_duration=f"{int(self.block_length_seconds)}s",
                use_full_vega_wallet=False,
            ) as vega:
                return self._run(vega, pause_at_completion=pause_at_completion)
        else:
            return self._run(self._vega, pause_at_completion=pause_at_completion)

    def _run(
        self,
        vega: VegaServiceNull,
        pause_at_completion: bool = False,
    ) -> Optional[List[Any]]:
        """Run the simulation with specified agents.

        Args:
            pause_at_completion:
                bool, default False, If True will pause with a keypress-prompt
                    once the simulation has completed, allowing the final state
                    to be inspected, either via code or the Console
        """
        logger.info(f"Running wallet at: {vega.wallet_url}")
        logger.info(
            f"Running graphql at: http://localhost:{vega.data_node_graphql_port}"
        )

        start = datetime.datetime.now()
        state_values = []

        for agent in self.agents:
            agent.initialise(vega=vega)
            if self.transactions_per_block > 1:
                vega.wait_fn(1)

        start_time = vega.get_blockchain_time()
        for i in range(self.n_steps):
            self.step(vega)

            # Ensure core is caught up
            core_catchup_start = datetime.datetime.now()
            vega.wait_for_core_catchup()
            core_catchup_seconds = (
                datetime.datetime.now() - core_catchup_start
            ).seconds

            if self.transactions_per_block > 1:
                vega.wait_fn(1)
            vega.wait_for_total_catchup()

            if core_catchup_seconds > 1:
                logger.warn(f"Waited {core_catchup_seconds}s for core catchup")

            if (
                self._state_extraction_fn is not None
                and i % self._state_extraction_freq == 0
            ):
                state_values.append(
                    self._state_extraction_fn(
                        vega, self.agents, state_values=state_values
                    )
                )

            vega.wait_for_total_catchup()

            if self.step_length_seconds is not None:
                end_time = vega.get_blockchain_time()
                to_forward = max(0, self.step_length_seconds - (end_time - start_time))
                if to_forward > 0:
                    logger.debug(
                        f"Forwarding {to_forward}s to round out the epoch, meaning"
                        " there were"
                        f" {(end_time - start_time) / self.block_length_seconds} blocks"
                        " produced this step"
                    )
                    vega.wait_fn(to_forward / self.block_length_seconds)
                start_time = vega.get_blockchain_time()

            if (
                self._pause_every_n_steps is not None
                and i % self._pause_every_n_steps == 0
            ):
                input(
                    f"Environment run at step {i}. Pausing to allow inspection of"
                    " state. Press Enter to continue"
                )

        logger.info(f"Run took {(datetime.datetime.now() - start).seconds}s")

        if pause_at_completion:
            input(
                "Environment run completed. Pausing to allow inspection of state."
                " Press Enter to continue"
            )
        for agent in self.agents:
            agent.finalise()
        vega.wait_for_core_catchup()
        vega.wait_for_datanode_sync()

        if self._state_extraction_fn is not None:
            state_values.append(
                self._state_extraction_fn(vega, self.agents, state_values=state_values)
            )
        if self._state_extraction_fn is not None:
            return state_values

    def step(self, vega: VegaService) -> None:
        for agent in (
            sorted(self.agents, key=lambda _: random.random())
            if self.random_agent_ordering
            else self.agents
        ):
            agent.step(vega)


class MarketEnvironmentWithState(MarketEnvironment):
    def __init__(
        self,
        agents: List[StateAgent],
        n_steps: int,
        random_agent_ordering: bool = True,
        state_func: Optional[Callable[[VegaService], VegaState]] = None,
        transactions_per_block: int = 1,
        block_length_seconds: int = 1,
        step_length_seconds: Optional[int] = None,
        vega_service: Optional[VegaServiceNull] = None,
        state_extraction_fn: Optional[
            Callable[[VegaServiceNull, List[Agent]], Any]
        ] = None,
        state_extraction_freq: int = 10,
        pause_every_n_steps: Optional[int] = None,
    ):
        """Set up a Vega protocol environment with some specified agents.
        Handles the entire Vega setup and environment lifetime process, allowing the
        user to focus on building the Agents themselves.

        Once an environment has been created, calling the 'run' function will
        run a complete simulation of the environment lifecycle then shut down
        the various Vega processes.

        This class differs from MarketEnvironment in that each agent will only
        receive a state representation object rather than a full VegaService.

        Args:
            agents:
                List[StateAgent], a list of instantiated Agent objects which will
                    interact with the environment
            n_steps:
                int, The number of steps for the environment to run
            random_agent_ordering:
                bool, default True, In each step, whether the order of agent
                    steps should be randomised. If False, the order of agents
                    passed in will be used
            transactions_per_block:
                int, default 1, How many transactions should be contained
                    for each block in the Vega chain. Often this is best set
                    as the maximum number of actions agents can take per step
                    to ensure they all happen 'at the same time' per step.
            block_length_seconds:
                int, default 1, How many seconds each block on the Vega chain
                    represents
            step_length_seconds:
                Optional[int], default None, How many seconds each step is
                    taken to represent.
                    After each round of actions, if time has not advanced at least
                    this much we will be forwarded to that far in the future
                    (minus however long the actions did take).
                    e.g. for a step_length_seconds = 60 if all actions take up 10s
                    we will forward 50s at the end.
            vega_service:
                optional, VegaServiceNull, If passed will use this precreated vega
                    service instead of creating one internally.
            state_extraction_fn:
                optional, Callable[[VegaServiceNull, List[Agent]], Any],
                    Optional function which takes a Vega service at a given time
                    and returns a summary of interesting state values. If passed,
                    the aggregated result of these will be returned after a run.
            state_extraction_freq:
                int, default 10, If state_extraction_fn is passed, how many steps
                    should be between each call.
            pause_every_n_steps:
                Optional[int], default None, If passed, simulation will pause every
                    time the passed number of steps elapses waiting on user to press
                    return. Allows inspection of the simulation at given frequency
        """
        super().__init__(
            agents=agents,
            n_steps=n_steps,
            random_agent_ordering=random_agent_ordering,
            transactions_per_block=transactions_per_block,
            block_length_seconds=block_length_seconds,
            step_length_seconds=step_length_seconds,
            vega_service=vega_service,
            state_extraction_fn=state_extraction_fn,
            state_extraction_freq=state_extraction_freq,
            pause_every_n_steps=pause_every_n_steps,
        )
        self.state_func = (
            state_func if state_func is not None else self._default_state_extraction
        )

    # @staticmethod
    def _default_state_extraction(self, vega: VegaService) -> VegaState:
        market_state = {}
        order_status = vega.order_status_from_feed(live_only=True)
        for market in vega.all_markets():
            market_info = vega.market_info(market_id=market.id)
            market_data = vega.market_data(market_id=market.id)
            market_state[market.id] = MarketState(
                state=market_info.state,
                trading_mode=market_info.trading_mode,
                midprice=float(market_data.mid_price)
                / 10 ** int(market_info.decimal_places),
                orders=order_status.get(market.id, {}),
            )

        return VegaState(network_state=(), market_state=market_state)

    def step(self, vega: VegaService) -> None:
        vega.wait_for_datanode_sync()
        state = self.state_func(vega)
        for agent in (
            sorted(self.agents, key=lambda _: random.random())
            if self.random_agent_ordering
            else self.agents
        ):
            agent.step(state)


class NetworkEnvironment(MarketEnvironmentWithState):
    def __init__(
        self,
        agents: List[StateAgent],
        n_steps: int = -1,
        step_length_seconds: int = 5,
        random_agent_ordering: bool = True,
        vega_service: Optional[VegaServiceNetwork] = None,
        state_func: Optional[Callable[[VegaService], VegaState]] = None,
        state_extraction_fn: Optional[
            Callable[[VegaServiceNetwork, List[Agent]], Any]
        ] = None,
        state_extraction_freq: int = 10,
        raise_datanode_errors: Optional[bool] = True,
        raise_step_errors: Optional[bool] = True,
    ):
        super().__init__(
            agents=agents,
            n_steps=n_steps,
            random_agent_ordering=random_agent_ordering,
            step_length_seconds=step_length_seconds,
            vega_service=vega_service,
            state_extraction_fn=state_extraction_fn,
            state_extraction_freq=state_extraction_freq,
        )
        self.state_func = (
            state_func if state_func is not None else self._default_state_extraction
        )

        self.raise_datanode_errors = raise_datanode_errors
        self.raise_step_errors = raise_step_errors

    def run(self):
        if self._vega is None:
            with VegaServiceNetwork(
                use_full_vega_wallet=True,
                run_with_wallet=True,
                run_with_console=True,
            ) as vega:
                return self._run(vega)
        else:
            return self._run(self._vega)

    def _run(self, vega: VegaServiceNetwork):
        state_values = []

        # Initial datanode connection check
        vega.check_datanode(raise_on_error=self.raise_datanode_errors)

        # Initialise agents without minting assets
        for agent in self.agents:
            agent.initialise(vega=vega, create_wallet=False, mint_wallet=False)

        i = 0
        # A negative self.n_steps will loop indefinitely
        while i != self.n_steps:

            vega.check_datanode(raise_on_error=self.raise_datanode_errors)

            i += 1
            self.step(vega)

            if (
                self._state_extraction_fn is not None
                and i % self._state_extraction_freq == 0
            ):
                state_values.append(self._state_extraction_fn(vega, self.agents))

            time.sleep(self.step_length_seconds)

        for agent in self.agents:
            agent.finalise()

        if self._state_extraction_fn is not None:
            state_values.append(self._state_extraction_fn(vega, self.agents))
            return state_values

    def step(self, vega: VegaServiceNetwork) -> None:
        t = time.time()
        state = self.state_func(vega)
        logging.debug(f"Get state took {time.time() - t} seconds.")
        for agent in (
            sorted(self.agents, key=lambda _: random.random())
            if self.random_agent_ordering
            else self.agents
        ):
            try:
                agent.step(state)
            except Exception as e:
                msg = f"Agent '{agent.key_name}' failed to step."
                if self.raise_step_errors:
                    raise e(msg)
                else:
                    logging.warning(msg)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from ...api.v1 import core_pb2 as vega_dot_api_dot_v1_dot_core__pb2


class CoreServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.SubmitTransaction = channel.unary_unary(
            "/vega.api.v1.CoreService/SubmitTransaction",
            request_serializer=vega_dot_api_dot_v1_dot_core__pb2.SubmitTransactionRequest.SerializeToString,
            response_deserializer=vega_dot_api_dot_v1_dot_core__pb2.SubmitTransactionResponse.FromString,
        )
        self.PropagateChainEvent = channel.unary_unary(
            "/vega.api.v1.CoreService/PropagateChainEvent",
            request_serializer=vega_dot_api_dot_v1_dot_core__pb2.PropagateChainEventRequest.SerializeToString,
            response_deserializer=vega_dot_api_dot_v1_dot_core__pb2.PropagateChainEventResponse.FromString,
        )
        self.Statistics = channel.unary_unary(
            "/vega.api.v1.CoreService/Statistics",
            request_serializer=vega_dot_api_dot_v1_dot_core__pb2.StatisticsRequest.SerializeToString,
            response_deserializer=vega_dot_api_dot_v1_dot_core__pb2.StatisticsResponse.FromString,
        )
        self.LastBlockHeight = channel.unary_unary(
            "/vega.api.v1.CoreService/LastBlockHeight",
            request_serializer=vega_dot_api_dot_v1_dot_core__pb2.LastBlockHeightRequest.SerializeToString,
            response_deserializer=vega_dot_api_dot_v1_dot_core__pb2.LastBlockHeightResponse.FromString,
        )
        self.GetVegaTime = channel.unary_unary(
            "/vega.api.v1.CoreService/GetVegaTime",
            request_serializer=vega_dot_api_dot_v1_dot_core__pb2.GetVegaTimeRequest.SerializeToString,
            response_deserializer=vega_dot_api_dot_v1_dot_core__pb2.GetVegaTimeResponse.FromString,
        )
        self.ObserveEventBus = channel.stream_stream(
            "/vega.api.v1.CoreService/ObserveEventBus",
            request_serializer=vega_dot_api_dot_v1_dot_core__pb2.ObserveEventBusRequest.SerializeToString,
            response_deserializer=vega_dot_api_dot_v1_dot_core__pb2.ObserveEventBusResponse.FromString,
        )
        self.SubmitRawTransaction = channel.unary_unary(
            "/vega.api.v1.CoreService/SubmitRawTransaction",
            request_serializer=vega_dot_api_dot_v1_dot_core__pb2.SubmitRawTransactionRequest.SerializeToString,
            response_deserializer=vega_dot_api_dot_v1_dot_core__pb2.SubmitRawTransactionResponse.FromString,
        )
        self.CheckTransaction = channel.unary_unary(
            "/vega.api.v1.CoreService/CheckTransaction",
            request_serializer=vega_dot_api_dot_v1_dot_core__pb2.CheckTransactionRequest.SerializeToString,
            response_deserializer=vega_dot_api_dot_v1_dot_core__pb2.CheckTransactionResponse.FromString,
        )
        self.CheckRawTransaction = channel.unary_unary(
            "/vega.api.v1.CoreService/CheckRawTransaction",
            request_serializer=vega_dot_api_dot_v1_dot_core__pb2.CheckRawTransactionRequest.SerializeToString,
            response_deserializer=vega_dot_api_dot_v1_dot_core__pb2.CheckRawTransactionResponse.FromString,
        )


class CoreServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def SubmitTransaction(self, request, context):
        """Submit Transaction

        Submit a signed transaction
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def PropagateChainEvent(self, request, context):
        """Chain event

        Propagate a chain event
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def Statistics(self, request, context):
        """Statistics

        Get Statistics on Vega
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def LastBlockHeight(self, request, context):
        """Blockchain height

        Get the height of the last tendermint block
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetVegaTime(self, request, context):
        """Vega time

        Get Time
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ObserveEventBus(self, request_iterator, context):
        """Subscribe to a stream of events from the core"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def SubmitRawTransaction(self, request, context):
        """Submit raw transaction

        Submit a version agnostic signed transaction
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def CheckTransaction(self, request, context):
        """Check transaction

        Check a signed transaction
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def CheckRawTransaction(self, request, context):
        """Check raw transaction

        Check a raw signed transaction
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_CoreServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "SubmitTransaction": grpc.unary_unary_rpc_method_handler(
            servicer.SubmitTransaction,
            request_deserializer=vega_dot_api_dot_v1_dot_core__pb2.SubmitTransactionRequest.FromString,
            response_serializer=vega_dot_api_dot_v1_dot_core__pb2.SubmitTransactionResponse.SerializeToString,
        ),
        "PropagateChainEvent": grpc.unary_unary_rpc_method_handler(
            servicer.PropagateChainEvent,
            request_deserializer=vega_dot_api_dot_v1_dot_core__pb2.PropagateChainEventRequest.FromString,
            response_serializer=vega_dot_api_dot_v1_dot_core__pb2.PropagateChainEventResponse.SerializeToString,
        ),
        "Statistics": grpc.unary_unary_rpc_method_handler(
            servicer.Statistics,
            request_deserializer=vega_dot_api_dot_v1_dot_core__pb2.StatisticsRequest.FromString,
            response_serializer=vega_dot_api_dot_v1_dot_core__pb2.StatisticsResponse.SerializeToString,
        ),
        "LastBlockHeight": grpc.unary_unary_rpc_method_handler(
            servicer.LastBlockHeight,
            request_deserializer=vega_dot_api_dot_v1_dot_core__pb2.LastBlockHeightRequest.FromString,
            response_serializer=vega_dot_api_dot_v1_dot_core__pb2.LastBlockHeightResponse.SerializeToString,
        ),
        "GetVegaTime": grpc.unary_unary_rpc_method_handler(
            servicer.GetVegaTime,
            request_deserializer=vega_dot_api_dot_v1_dot_core__pb2.GetVegaTimeRequest.FromString,
            response_serializer=vega_dot_api_dot_v1_dot_core__pb2.GetVegaTimeResponse.SerializeToString,
        ),
        "ObserveEventBus": grpc.stream_stream_rpc_method_handler(
            servicer.ObserveEventBus,
            request_deserializer=vega_dot_api_dot_v1_dot_core__pb2.ObserveEventBusRequest.FromString,
            response_serializer=vega_dot_api_dot_v1_dot_core__pb2.ObserveEventBusResponse.SerializeToString,
        ),
        "SubmitRawTransaction": grpc.unary_unary_rpc_method_handler(
            servicer.SubmitRawTransaction,
            request_deserializer=vega_dot_api_dot_v1_dot_core__pb2.SubmitRawTransactionRequest.FromString,
            response_serializer=vega_dot_api_dot_v1_dot_core__pb2.SubmitRawTransactionResponse.SerializeToString,
        ),
        "CheckTransaction": grpc.unary_unary_rpc_method_handler(
            servicer.CheckTransaction,
            request_deserializer=vega_dot_api_dot_v1_dot_core__pb2.CheckTransactionRequest.FromString,
            response_serializer=vega_dot_api_dot_v1_dot_core__pb2.CheckTransactionResponse.SerializeToString,
        ),
        "CheckRawTransaction": grpc.unary_unary_rpc_method_handler(
            servicer.CheckRawTransaction,
            request_deserializer=vega_dot_api_dot_v1_dot_core__pb2.CheckRawTransactionRequest.FromString,
            response_serializer=vega_dot_api_dot_v1_dot_core__pb2.CheckRawTransactionResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "vega.api.v1.CoreService", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class CoreService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def SubmitTransaction(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/vega.api.v1.CoreService/SubmitTransaction",
            vega_dot_api_dot_v1_dot_core__pb2.SubmitTransactionRequest.SerializeToString,
            vega_dot_api_dot_v1_dot_core__pb2.SubmitTransactionResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def PropagateChainEvent(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/vega.api.v1.CoreService/PropagateChainEvent",
            vega_dot_api_dot_v1_dot_core__pb2.PropagateChainEventRequest.SerializeToString,
            vega_dot_api_dot_v1_dot_core__pb2.PropagateChainEventResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def Statistics(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/vega.api.v1.CoreService/Statistics",
            vega_dot_api_dot_v1_dot_core__pb2.StatisticsRequest.SerializeToString,
            vega_dot_api_dot_v1_dot_core__pb2.StatisticsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def LastBlockHeight(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/vega.api.v1.CoreService/LastBlockHeight",
            vega_dot_api_dot_v1_dot_core__pb2.LastBlockHeightRequest.SerializeToString,
            vega_dot_api_dot_v1_dot_core__pb2.LastBlockHeightResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def GetVegaTime(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/vega.api.v1.CoreService/GetVegaTime",
            vega_dot_api_dot_v1_dot_core__pb2.GetVegaTimeRequest.SerializeToString,
            vega_dot_api_dot_v1_dot_core__pb2.GetVegaTimeResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ObserveEventBus(
        request_iterator,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.stream_stream(
            request_iterator,
            target,
            "/vega.api.v1.CoreService/ObserveEventBus",
            vega_dot_api_dot_v1_dot_core__pb2.ObserveEventBusRequest.SerializeToString,
            vega_dot_api_dot_v1_dot_core__pb2.ObserveEventBusResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def SubmitRawTransaction(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/vega.api.v1.CoreService/SubmitRawTransaction",
            vega_dot_api_dot_v1_dot_core__pb2.SubmitRawTransactionRequest.SerializeToString,
            vega_dot_api_dot_v1_dot_core__pb2.SubmitRawTransactionResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def CheckTransaction(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/vega.api.v1.CoreService/CheckTransaction",
            vega_dot_api_dot_v1_dot_core__pb2.CheckTransactionRequest.SerializeToString,
            vega_dot_api_dot_v1_dot_core__pb2.CheckTransactionResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def CheckRawTransaction(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/vega.api.v1.CoreService/CheckRawTransaction",
            vega_dot_api_dot_v1_dot_core__pb2.CheckRawTransactionRequest.SerializeToString,
            vega_dot_api_dot_v1_dot_core__pb2.CheckRawTransactionResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



import logging
from vega_sim.reinforcement.agents.learning_agent import (
    WALLET as LEARNING_WALLET,
)

from vega_sim.reinforcement.agents.simple_agent import SimpleAgent

from vega_sim.scenario.registry import CurveMarketMaker

from vega_sim.reinforcement.helpers import set_seed
from vega_sim.null_service import VegaServiceNull


def run_iteration(
    learning_agent: SimpleAgent,
    vega,
    market_name: str,
    asset_name: str,
    run_with_console=False,
    pause_at_completion=False,
):
    scenario = CurveMarketMaker(
        market_decimal=3,
        asset_decimal=5,
        market_position_decimal=2,
        initial_price=1000.0,
        lp_commitamount=100000,
        initial_asset_mint=1e8,
        step_length_seconds=1,
        block_length_seconds=1,
        buy_intensity=5,
        sell_intensity=5,
        market_name=market_name,
        asset_name=asset_name,
        num_steps=50,
        random_agent_ordering=False,
        sigma=100,
    )
    env = scenario.set_up_background_market(
        vega=vega,
    )
    # add the learning agaent to the environement's list of agents
    env.agents = env.agents + [learning_agent]

    learning_agent.price_process = scenario.price_process

    result = env.run(
        run_with_console=run_with_console,
        pause_at_completion=pause_at_completion,
    )
    return result


if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)

    # set seed for results replication
    set_seed(1)

    # set market name
    market_name = "ETH:USD"
    asset_name = "tDAI"

    # create the Learning Agent
    agent = SimpleAgent(
        wallet_name=LEARNING_WALLET.name,
        wallet_pass=LEARNING_WALLET.passphrase,
        initial_balance=100_000,
        market_name=market_name,
        asset_name=asset_name,
    )

    with VegaServiceNull(
        warn_on_raw_data_access=False, run_with_console=False, retain_log_files=True
    ) as vega:
        _ = run_iteration(
            learning_agent=agent,
            vega=vega,
            market_name=market_name,
            pause_at_completion=True,
            asset_name=asset_name,
        )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: vega/checkpoint/v1/checkpoint.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ... import vega_pb2 as vega_dot_vega__pb2
from ... import assets_pb2 as vega_dot_assets__pb2
from ... import governance_pb2 as vega_dot_governance__pb2
from ...events.v1 import events_pb2 as vega_dot_events_dot_v1_dot_events__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n#vega/checkpoint/v1/checkpoint.proto\x12\x12vega.checkpoint.v1\x1a\x0fvega/vega.proto\x1a\x11vega/assets.proto\x1a\x15vega/governance.proto\x1a\x1bvega/events/v1/events.proto";\n\x0f\x43heckpointState\x12\x12\n\x04hash\x18\x01 \x01(\x0cR\x04hash\x12\x14\n\x05state\x18\x02 \x01(\x0cR\x05state"\x9f\x03\n\nCheckpoint\x12\x1e\n\ngovernance\x18\x01 \x01(\x0cR\ngovernance\x12\x16\n\x06\x61ssets\x18\x02 \x01(\x0cR\x06\x61ssets\x12\x1e\n\ncollateral\x18\x03 \x01(\x0cR\ncollateral\x12-\n\x12network_parameters\x18\x04 \x01(\x0cR\x11networkParameters\x12\x1e\n\ndelegation\x18\x05 \x01(\x0cR\ndelegation\x12\x14\n\x05\x65poch\x18\x06 \x01(\x0cR\x05\x65poch\x12\x14\n\x05\x62lock\x18\x07 \x01(\x0cR\x05\x62lock\x12\x18\n\x07rewards\x18\x08 \x01(\x0cR\x07rewards\x12\x18\n\x07\x62\x61nking\x18\t \x01(\x0cR\x07\x62\x61nking\x12\x1e\n\nvalidators\x18\n \x01(\x0cR\nvalidators\x12\x18\n\x07staking\x18\x0b \x01(\x0cR\x07staking\x12)\n\x10multisig_control\x18\x0c \x01(\x0cR\x0fmultisigControl\x12%\n\x0emarket_tracker\x18\r \x01(\x0cR\rmarketTracker"U\n\nAssetEntry\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x37\n\rasset_details\x18\x02 \x01(\x0b\x32\x12.vega.AssetDetailsR\x0c\x61ssetDetails"\x96\x01\n\x06\x41ssets\x12\x36\n\x06\x61ssets\x18\x01 \x03(\x0b\x32\x1e.vega.checkpoint.v1.AssetEntryR\x06\x61ssets\x12T\n\x16pending_listing_assets\x18\x02 \x03(\x0b\x32\x1e.vega.checkpoint.v1.AssetEntryR\x14pendingListingAssets"T\n\x0c\x41ssetBalance\x12\x14\n\x05party\x18\x01 \x01(\tR\x05party\x12\x14\n\x05\x61sset\x18\x02 \x01(\tR\x05\x61sset\x12\x18\n\x07\x62\x61lance\x18\x03 \x01(\tR\x07\x62\x61lance"J\n\nCollateral\x12<\n\x08\x62\x61lances\x18\x01 \x03(\x0b\x32 .vega.checkpoint.v1.AssetBalanceR\x08\x62\x61lances";\n\tNetParams\x12.\n\x06params\x18\x01 \x03(\x0b\x32\x16.vega.NetworkParameterR\x06params"9\n\tProposals\x12,\n\tproposals\x18\x01 \x03(\x0b\x32\x0e.vega.ProposalR\tproposals"\x8e\x01\n\rDelegateEntry\x12\x14\n\x05party\x18\x01 \x01(\tR\x05party\x12\x12\n\x04node\x18\x02 \x01(\tR\x04node\x12\x16\n\x06\x61mount\x18\x03 \x01(\tR\x06\x61mount\x12\x1e\n\nundelegate\x18\x04 \x01(\x08R\nundelegate\x12\x1b\n\tepoch_seq\x18\x05 \x01(\x04R\x08\x65pochSeq"\xab\x01\n\x08\x44\x65legate\x12\x39\n\x06\x61\x63tive\x18\x01 \x03(\x0b\x32!.vega.checkpoint.v1.DelegateEntryR\x06\x61\x63tive\x12;\n\x07pending\x18\x02 \x03(\x0b\x32!.vega.checkpoint.v1.DelegateEntryR\x07pending\x12\'\n\x0f\x61uto_delegation\x18\x03 \x03(\tR\x0e\x61utoDelegation"\x1f\n\x05\x42lock\x12\x16\n\x06height\x18\x01 \x01(\x03R\x06height"E\n\x07Rewards\x12:\n\x07rewards\x18\x01 \x03(\x0b\x32 .vega.checkpoint.v1.RewardPayoutR\x07rewards"\x7f\n\x0cRewardPayout\x12\x1f\n\x0bpayout_time\x18\x01 \x01(\x03R\npayoutTime\x12N\n\x0erewards_payout\x18\x02 \x03(\x0b\x32\'.vega.checkpoint.v1.PendingRewardPayoutR\rrewardsPayout"\xf0\x01\n\x13PendingRewardPayout\x12!\n\x0c\x66rom_account\x18\x01 \x01(\tR\x0b\x66romAccount\x12\x14\n\x05\x61sset\x18\x02 \x01(\tR\x05\x61sset\x12\x42\n\x0cparty_amount\x18\x03 \x03(\x0b\x32\x1f.vega.checkpoint.v1.PartyAmountR\x0bpartyAmount\x12!\n\x0ctotal_reward\x18\x04 \x01(\tR\x0btotalReward\x12\x1b\n\tepoch_seq\x18\x05 \x01(\tR\x08\x65pochSeq\x12\x1c\n\ttimestamp\x18\x06 \x01(\x03R\ttimestamp";\n\x0bPartyAmount\x12\x14\n\x05party\x18\x01 \x01(\tR\x05party\x12\x16\n\x06\x61mount\x18\x02 \x01(\tR\x06\x61mount"\xb9\x01\n\x12PendingKeyRotation\x12?\n\x1crelative_target_block_height\x18\x01 \x01(\x04R\x19relativeTargetBlockHeight\x12\x17\n\x07node_id\x18\x02 \x01(\tR\x06nodeId\x12\x1e\n\x0bnew_pub_key\x18\x03 \x01(\tR\tnewPubKey\x12)\n\x11new_pub_key_index\x18\x04 \x01(\rR\x0enewPubKeyIndex"\x97\x01\n\x1aPendingEthereumKeyRotation\x12?\n\x1crelative_target_block_height\x18\x01 \x01(\x04R\x19relativeTargetBlockHeight\x12\x17\n\x07node_id\x18\x02 \x01(\tR\x06nodeId\x12\x1f\n\x0bnew_address\x18\x03 \x01(\tR\nnewAddress"\xd6\x01\n\x11ScheduledTransfer\x12*\n\x08transfer\x18\x01 \x01(\x0b\x32\x0e.vega.TransferR\x08transfer\x12\x34\n\x0c\x61\x63\x63ount_type\x18\x02 \x01(\x0e\x32\x11.vega.AccountTypeR\x0b\x61\x63\x63ountType\x12\x1c\n\treference\x18\x03 \x01(\tR\treference\x12\x41\n\x0foneoff_transfer\x18\x04 \x01(\x0b\x32\x18.vega.events.v1.TransferR\x0eoneoffTransfer"}\n\x17ScheduledTransferAtTime\x12\x1d\n\ndeliver_on\x18\x01 \x01(\x03R\tdeliverOn\x12\x43\n\ttransfers\x18\x02 \x03(\x0b\x32%.vega.checkpoint.v1.ScheduledTransferR\ttransfers"_\n\x12RecurringTransfers\x12I\n\x13recurring_transfers\x18\x01 \x03(\x0b\x32\x18.vega.events.v1.TransferR\x12recurringTransfers"\xff\x01\n\x07\x42\x61nking\x12W\n\x11transfers_at_time\x18\x01 \x03(\x0b\x32+.vega.checkpoint.v1.ScheduledTransferAtTimeR\x0ftransfersAtTime\x12W\n\x13recurring_transfers\x18\x02 \x01(\x0b\x32&.vega.checkpoint.v1.RecurringTransfersR\x12recurringTransfers\x12\x42\n\x0c\x62ridge_state\x18\x03 \x01(\x0b\x32\x1f.vega.checkpoint.v1.BridgeStateR\x0b\x62ridgeState"e\n\x0b\x42ridgeState\x12\x16\n\x06\x61\x63tive\x18\x01 \x01(\x08R\x06\x61\x63tive\x12!\n\x0c\x62lock_height\x18\x02 \x01(\x04R\x0b\x62lockHeight\x12\x1b\n\tlog_index\x18\x03 \x01(\x04R\x08logIndex"\xaa\x02\n\nValidators\x12K\n\x0fvalidator_state\x18\x01 \x03(\x0b\x32".vega.checkpoint.v1.ValidatorStateR\x0evalidatorState\x12Z\n\x15pending_key_rotations\x18\x02 \x03(\x0b\x32&.vega.checkpoint.v1.PendingKeyRotationR\x13pendingKeyRotations\x12s\n\x1epending_ethereum_key_rotations\x18\x03 \x03(\x0b\x32..vega.checkpoint.v1.PendingEthereumKeyRotationR\x1bpendingEthereumKeyRotations"\x88\x02\n\x0eValidatorState\x12J\n\x10validator_update\x18\x01 \x01(\x0b\x32\x1f.vega.events.v1.ValidatorUpdateR\x0fvalidatorUpdate\x12\x16\n\x06status\x18\x02 \x01(\x05R\x06status\x12\x30\n\x14\x65th_events_forwarded\x18\x03 \x01(\x04R\x12\x65thEventsForwarded\x12\'\n\x0fvalidator_power\x18\x04 \x01(\x03R\x0evalidatorPower\x12\x37\n\rranking_score\x18\x05 \x01(\x0b\x32\x12.vega.RankingScoreR\x0crankingScore"k\n\x07Staking\x12\x38\n\x08\x61\x63\x63\x65pted\x18\x01 \x03(\x0b\x32\x1c.vega.events.v1.StakeLinkingR\x08\x61\x63\x63\x65pted\x12&\n\x0flast_block_seen\x18\x02 \x01(\x04R\rlastBlockSeen"\xd2\x01\n\x0fMultisigControl\x12\x42\n\x07signers\x18\x01 \x03(\x0b\x32(.vega.events.v1.ERC20MultiSigSignerEventR\x07signers\x12S\n\rthreshold_set\x18\x02 \x01(\x0b\x32..vega.events.v1.ERC20MultiSigThresholdSetEventR\x0cthresholdSet\x12&\n\x0flast_block_seen\x18\x03 \x01(\x04R\rlastBlockSeen"c\n\rMarketTracker\x12R\n\x0fmarket_activity\x18\x01 \x03(\x0b\x32).vega.checkpoint.v1.MarketActivityTrackerR\x0emarketActivity"\x99\x03\n\x15MarketActivityTracker\x12\x16\n\x06market\x18\x01 \x01(\tR\x06market\x12\x14\n\x05\x61sset\x18\x02 \x01(\tR\x05\x61sset\x12M\n\x13maker_fees_received\x18\x03 \x03(\x0b\x32\x1d.vega.checkpoint.v1.PartyFeesR\x11makerFeesReceived\x12\x45\n\x0fmaker_fees_paid\x18\x04 \x03(\x0b\x32\x1d.vega.checkpoint.v1.PartyFeesR\rmakerFeesPaid\x12\x36\n\x07lp_fees\x18\x05 \x03(\x0b\x32\x1d.vega.checkpoint.v1.PartyFeesR\x06lpFees\x12\x1a\n\x08proposer\x18\x06 \x01(\tR\x08proposer\x12\x1d\n\nbonus_paid\x18\x07 \x03(\tR\tbonusPaid\x12!\n\x0cvalue_traded\x18\x08 \x01(\tR\x0bvalueTraded\x12&\n\x0fready_to_delete\x18\t \x01(\x08R\rreadyToDelete"3\n\tPartyFees\x12\x14\n\x05party\x18\x01 \x01(\tR\x05party\x12\x10\n\x03\x66\x65\x65\x18\x02 \x01(\tR\x03\x66\x65\x65\x42\x35Z3code.vegaprotocol.io/vega/protos/vega/checkpoint/v1b\x06proto3'
)

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(
    DESCRIPTOR, "vega.checkpoint.v1.checkpoint_pb2", globals()
)
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    DESCRIPTOR._serialized_options = (
        b"Z3code.vegaprotocol.io/vega/protos/vega/checkpoint/v1"
    )
    _CHECKPOINTSTATE._serialized_start = 147
    _CHECKPOINTSTATE._serialized_end = 206
    _CHECKPOINT._serialized_start = 209
    _CHECKPOINT._serialized_end = 624
    _ASSETENTRY._serialized_start = 626
    _ASSETENTRY._serialized_end = 711
    _ASSETS._serialized_start = 714
    _ASSETS._serialized_end = 864
    _ASSETBALANCE._serialized_start = 866
    _ASSETBALANCE._serialized_end = 950
    _COLLATERAL._serialized_start = 952
    _COLLATERAL._serialized_end = 1026
    _NETPARAMS._serialized_start = 1028
    _NETPARAMS._serialized_end = 1087
    _PROPOSALS._serialized_start = 1089
    _PROPOSALS._serialized_end = 1146
    _DELEGATEENTRY._serialized_start = 1149
    _DELEGATEENTRY._serialized_end = 1291
    _DELEGATE._serialized_start = 1294
    _DELEGATE._serialized_end = 1465
    _BLOCK._serialized_start = 1467
    _BLOCK._serialized_end = 1498
    _REWARDS._serialized_start = 1500
    _REWARDS._serialized_end = 1569
    _REWARDPAYOUT._serialized_start = 1571
    _REWARDPAYOUT._serialized_end = 1698
    _PENDINGREWARDPAYOUT._serialized_start = 1701
    _PENDINGREWARDPAYOUT._serialized_end = 1941
    _PARTYAMOUNT._serialized_start = 1943
    _PARTYAMOUNT._serialized_end = 2002
    _PENDINGKEYROTATION._serialized_start = 2005
    _PENDINGKEYROTATION._serialized_end = 2190
    _PENDINGETHEREUMKEYROTATION._serialized_start = 2193
    _PENDINGETHEREUMKEYROTATION._serialized_end = 2344
    _SCHEDULEDTRANSFER._serialized_start = 2347
    _SCHEDULEDTRANSFER._serialized_end = 2561
    _SCHEDULEDTRANSFERATTIME._serialized_start = 2563
    _SCHEDULEDTRANSFERATTIME._serialized_end = 2688
    _RECURRINGTRANSFERS._serialized_start = 2690
    _RECURRINGTRANSFERS._serialized_end = 2785
    _BANKING._serialized_start = 2788
    _BANKING._serialized_end = 3043
    _BRIDGESTATE._serialized_start = 3045
    _BRIDGESTATE._serialized_end = 3146
    _VALIDATORS._serialized_start = 3149
    _VALIDATORS._serialized_end = 3447
    _VALIDATORSTATE._serialized_start = 3450
    _VALIDATORSTATE._serialized_end = 3714
    _STAKING._serialized_start = 3716
    _STAKING._serialized_end = 3823
    _MULTISIGCONTROL._serialized_start = 3826
    _MULTISIGCONTROL._serialized_end = 4036
    _MARKETTRACKER._serialized_start = 4038
    _MARKETTRACKER._serialized_end = 4137
    _MARKETACTIVITYTRACKER._serialized_start = 4140
    _MARKETACTIVITYTRACKER._serialized_end = 4549
    _PARTYFEES._serialized_start = 4551
    _PARTYFEES._serialized_end = 4602
# @@protoc_insertion_point(module_scope)



from vega_sim.api.market import MarketConfig
from vega_sim.scenario.common.agents import ShapedMarketMaker
from vega_sim.scenario.comprehensive_market.scenario import ComprehensiveMarket
from vega_sim.scenario.curve_market_maker.scenario import CurveMarketMaker
from vega_sim.scenario.ideal_market_maker.scenario import IdealMarketMaker
from vega_sim.scenario.ideal_market_maker_v2.scenario import (
    IdealMarketMaker as IdealMarketMakerV2,
)
from vega_sim.scenario.multi_market.scenario import VegaLoadTest
from vega_sim.scenario.market_crash.scenario import MarketCrash
from vega_sim.scenario.configurable_market.scenario import ConfigurableMarket

from vega_sim.scenario.common.utils.price_process import (
    get_historic_price_series,
    Granularity,
)

SCENARIOS = {
    "comprehensive_market": lambda: ComprehensiveMarket(
        market_name="ETH",
        asset_name="USD",
        num_steps=12000,
        market_decimal=2,
        asset_decimal=4,
        market_position_decimal=4,
        initial_price=1000.00,
        spread=10,
        lp_commitamount=500_000,
        initial_asset_mint=10_000_000,
        step_length_seconds=60,
        block_length_seconds=1,
        opening_auction_trade_amount=1,
        market_order_trader_order_intensity=10,
        market_order_trader_order_size=0.01,
        limit_order_trader_quantity=5,
        limit_order_trader_submit_bias=0.1,
        limit_order_trader_cancel_bias=0.1,
        limit_order_trader_order_intensity=10,
        limit_order_trader_order_size=0.1,
        limit_order_trader_mean=-5,
        limit_order_trader_sigma=0.5,
        limit_order_trader_duration=300,
        limit_order_trader_time_in_force_opts={
            "TIME_IN_FORCE_GTC": 0.7,
            "TIME_IN_FORCE_GTT": 0.3,
        },
        num_lp_agents=3,
        num_mo_agents=5,
        num_lo_agents=20,
    ),
    "ideal_market_maker": IdealMarketMaker,
    "ideal_market_maker_v2": lambda: IdealMarketMakerV2(
        num_steps=2000,
        market_decimal=3,
        asset_decimal=5,
        market_position_decimal=2,
        initial_price=1123.11,
        spread=4,
        lp_commitamount=1000000,
        initial_asset_mint=1e8,
        step_length_seconds=60,
        block_length_seconds=1,
        buy_intensity=10,
        sell_intensity=10,
        q_upper=50,
        q_lower=-50,
        kappa=50,
        sigma=5,
        backgroundmarket_tick_spacing=0.002,
        backgroundmarket_number_levels_per_side=25,
        settle_at_end=False,
    ),
    "market_crash": lambda: MarketCrash(
        num_steps=500,
        sigma_pre=1,
        sigma_post=4,
        drift_pre=0.1,
        drift_post=-0.5,
        break_point=200,
        initial_price=100,
        kappa=1.1,
        position_taker_buy_intensity=3,
        position_taker_sell_intensity=0,
        noise_buy_intensity=3,
        noise_sell_intensity=3,
        num_position_traders=5,
        num_noise_traders=20,
        step_length_seconds=60,
        block_length_seconds=1,
        trim_to_min=1,
    ),
    "historic_ideal_market_maker_v2": lambda: IdealMarketMakerV2(
        market_name="ETH",
        asset_name="USD",
        num_steps=290,
        market_decimal=2,
        asset_decimal=4,
        market_position_decimal=4,
        price_process_fn=lambda: get_historic_price_series(
            product_id="ETH-USD", granularity=Granularity.HOUR
        ).values,
        spread=0.01,
        lp_commitamount=250_000,
        initial_asset_mint=10_000_000,
        step_length_seconds=60,
        # step_length_seconds=Granularity.HOUR.value,
        block_length_seconds=1,
        buy_intensity=700_000,
        sell_intensity=700_000,
        q_upper=2,
        q_lower=-2,
        kappa=0.2,
        opening_auction_trade_amount=0.0001,
        backgroundmarket_tick_spacing=0.1,
        backgroundmarket_number_levels_per_side=25,
        market_order_trader_base_order_size=0.01,
    ),
    "historic_shaped_market_maker": lambda: CurveMarketMaker(
        market_name="ETH",
        asset_name="USD",
        num_steps=290,
        market_decimal=2,
        asset_decimal=4,
        market_position_decimal=4,
        price_process_fn=lambda: get_historic_price_series(
            product_id="ETH-USD", granularity=Granularity.HOUR
        ).values,
        lp_commitamount=250_000,
        initial_asset_mint=10_000_000,
        step_length_seconds=60,
        # step_length_seconds=Granularity.HOUR.value,
        block_length_seconds=1,
        q_upper=30,
        q_lower=-30,
        market_maker_curve_kappa=0.2,
        market_maker_assumed_market_kappa=0.2,
        buy_intensity=100,
        sell_intensity=100,
        sensitive_price_taker_half_life=10,
        opening_auction_trade_amount=0.0001,
        market_order_trader_base_order_size=0.01,
    ),
    "configurable_market": lambda: ConfigurableMarket(
        market_name="RESEARCH: Ethereum:USD Q3 (Daily)",
        market_code="ETH:USD",
        asset_name="tUSD",
        asset_dp=18,
        num_steps=60 * 24,
    ),
    "vega_load_test": lambda: VegaLoadTest(
        num_steps=15 * 24 * 30,
        granularity=Granularity.FIFTEEN_MINUTE,
        block_length_seconds=60,
        transactions_per_block=4000,
        parties_per_market=200,
        orders_per_second=100,
        trades_per_second=1,
    ),
}
